@@ -38,6 +38,41 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define SPECIAL_HIGHRES_BUILD_CRF 28
 
+
+#if defined(ANDROID) || (TARGET_OS_IPHONE == 1) || defined(__arm__)
+	#define MS_X264_CONF(required_bitrate, bitrate_limit, resolution, fps_pc, cpus_pc, fps_mobile, cpus_mobile) \
+		{ required_bitrate, bitrate_limit, { MS_VIDEO_SIZE_ ## resolution ## _W, MS_VIDEO_SIZE_ ## resolution ## _H },fps_pc, cpus_pc, NULL }
+#else
+	#define MS_X264_CONF(required_bitrate, bitrate_limit, resolution, fps_pc, cpus_pc, fps_mobile, cpus_mobile) \
+		{ required_bitrate, bitrate_limit, { MS_VIDEO_SIZE_ ## resolution ## _W, MS_VIDEO_SIZE_ ## resolution ## _H },fps_mobile, cpus_mobile, NULL }
+#endif
+
+
+
+static const MSVideoConfiguration x264_conf_list[] = {
+	MS_X264_CONF(2048000, 	1000000,            UXGA, 25,  4, 12, 2), /*1200p*/
+
+	MS_X264_CONF(1024000, 	5000000, 	  SXGA_MINUS, 25,  4, 12, 2),
+
+	MS_X264_CONF(1024000,  	5000000,   			720P, 25,  4, 12, 2),
+
+	MS_X264_CONF( 750000, 	2048000,             XGA, 20,  4, 12, 2),
+
+	MS_X264_CONF( 500000,  	1024000,            SVGA, 20,  2, 12, 2),
+
+	MS_X264_CONF( 256000,  	 800000,             VGA, 15,  2, 12, 2), /*480p*/
+
+	MS_X264_CONF( 128000,  	 512000,             CIF, 15,  1, 12, 2),
+
+	MS_X264_CONF( 100000,  	 380000,            QVGA, 15,  1, 15, 2), /*240p*/
+	MS_X264_CONF( 100000,  	 380000,            QVGA, 15,  1, 12, 1),
+
+	MS_X264_CONF( 128000,    170000,            QCIF, 10,  1, 10, 1),
+	MS_X264_CONF(  64000,    128000,            QCIF, 10,  1, 7, 1),
+	MS_X264_CONF(      0,     64000,            QCIF, 10,  1, 5, 1)
+};
+
+
 /* the goal of this small object is to tell when to send I frames at startup:
 at 2 and 4 seconds*/
 typedef struct VideoStarter{
@@ -71,14 +106,13 @@ static bool_t video_starter_need_i_frame(VideoStarter *vs, uint64_t curtime){
 typedef struct _EncData{
 	x264_t *enc;
 	x264_param_t params;
-	MSVideoSize vsize;
-	int bitrate;
-	float fps;
 	int mode;
 	uint64_t framenum;
 	Rfc3984Context *packer;
 	int keyframe_int;
 	VideoStarter starter;
+	const MSVideoConfiguration *vconf_list;
+	MSVideoConfiguration vconf;
 	bool_t generate_keyframe;
 }EncData;
 
@@ -86,14 +120,13 @@ typedef struct _EncData{
 static void enc_init(MSFilter *f){
 	EncData *d=ms_new(EncData,1);
 	d->enc=NULL;
-    MS_VIDEO_SIZE_ASSIGN(d->vsize,CIF);
-	d->bitrate=384000;
-	d->fps=30;
 	d->keyframe_int=10; /*10 seconds */
 	d->mode=0;
 	d->framenum=0;
 	d->generate_keyframe=FALSE;
 	d->packer=NULL;
+	d->vconf_list=x264_conf_list;
+	d->vconf = ms_video_find_best_configuration_for_bitrate(d->vconf_list, 384000, ms_get_cpu_count());
 	f->data=d;
 }
 
@@ -102,13 +135,12 @@ static void enc_uninit(MSFilter *f){
 	ms_free(d);
 }
 
-static void apply_bitrate(MSFilter *f, int target_bitrate){
+static void apply_bitrate(MSFilter *f){
 	EncData *d=(EncData*)f->data;
 	x264_param_t *params=&d->params;
 	float bitrate;
 
-	d->bitrate=target_bitrate;
-	bitrate=(float)d->bitrate*0.92;
+	bitrate=(float)d->vconf.required_bitrate*0.92;
 	if (bitrate>RC_MARGIN)
 		bitrate-=RC_MARGIN;
 	
@@ -127,24 +159,24 @@ static void enc_preprocess(MSFilter *f){
 	d->packer=rfc3984_new();
 	rfc3984_set_mode(d->packer,d->mode);
 	rfc3984_enable_stap_a(d->packer,FALSE);
-#ifdef __arm__	
-	if (x264_param_default_preset(params,"superfast"/*"ultrafast"*/,"zerolatency")) { 
-#else
-		x264_param_default(params); {
-#endif
+#if defined(__arm__) || defined(ANDROID)
+	if (x264_param_default_preset(params,"superfast"/*"ultrafast"*/,"zerolatency")) {
 		ms_error("Cannot apply default x264 configuration");
-	};
+	}
+#else
+	x264_param_default(params);
+#endif
 	
-	params->i_threads=1;
+	params->i_threads=ms_get_cpu_count();
 	params->i_sync_lookahead=0;
-	params->i_width=d->vsize.width;
-	params->i_height=d->vsize.height;
-	params->i_fps_num=(int)d->fps;
+	params->i_width=d->vconf.vsize.width;
+	params->i_height=d->vconf.vsize.height;
+	params->i_fps_num=(int)d->vconf.fps;
 	params->i_fps_den=1;
 	params->i_slice_max_size=ms_get_payload_max_size()-100; /*-100 security margin*/
 	params->i_level_idc=13;
 	
-	apply_bitrate(f,d->bitrate);
+	apply_bitrate(f);
 
 	params->rc.i_lookahead=0;
 	/*enable this by config ?*/
@@ -191,6 +223,12 @@ static void enc_process(MSFilter *f){
 	mblk_t *im;
 	MSPicture pic;
 	MSQueue nalus;
+	
+	if (d->enc==NULL){
+		ms_queue_flush(f->inputs[0]);
+		return;
+	}
+	
 	ms_queue_init(&nalus);
 	while((im=ms_queue_get(f->inputs[0]))!=NULL){
 		if (ms_yuv_buf_init_from_mblk(&pic,im)==0){
@@ -223,7 +261,7 @@ static void enc_process(MSFilter *f){
 			xpic.img.plane[1]=pic.planes[1];
 			xpic.img.plane[2]=pic.planes[2];
 			xpic.img.plane[3]=0;
-            
+
 			if (x264_encoder_encode(d->enc,&xnals,&num_nals,&xpic,&oxpic)>=0){
 				x264_nals_to_msgb(xnals,num_nals,&nalus);
 				/*if (num_nals == 0)	ms_message("Delayed frames info: current=%d max=%d\n", 
@@ -254,88 +292,73 @@ static void enc_postprocess(MSFilter *f){
 
 static int enc_get_br(MSFilter *f, void*arg){
 	EncData *d=(EncData*)f->data;
-	*(int*)arg=d->bitrate;
+	*(int*)arg=d->vconf.required_bitrate;
 	return 0;
 }
 
-static int enc_set_br(MSFilter *f, void *arg){
-	EncData *d=(EncData*)f->data;
-	d->bitrate=*(int*)arg;
+static int enc_set_configuration(MSFilter *f, void *arg) {
+	EncData *d = (EncData *)f->data;
+	const MSVideoConfiguration *vconf = (const MSVideoConfiguration *)arg;
+	if (vconf != &d->vconf) memcpy(&d->vconf, vconf, sizeof(MSVideoConfiguration));
 
-	if (d->enc){
+	if (d->vconf.required_bitrate > d->vconf.bitrate_limit)
+		d->vconf.required_bitrate = d->vconf.bitrate_limit;
+	if (d->enc) {
 		ms_filter_lock(f);
-		apply_bitrate(f,d->bitrate);
-		if (x264_encoder_reconfig(d->enc,&d->params)!=0){
+		apply_bitrate(f);
+		if (x264_encoder_reconfig(d->enc, &d->params) != 0) {
 			ms_error("x264_encoder_reconfig() failed.");
 		}
 		ms_filter_unlock(f);
 		return 0;
 	}
-	
-	if (d->bitrate>=1024000){
-		d->vsize.width = MS_VIDEO_SIZE_SVGA_W;
-		d->vsize.height = MS_VIDEO_SIZE_SVGA_H;
-		d->fps=25;
-	}else if (d->bitrate>=512000){
-		d->vsize.width = MS_VIDEO_SIZE_VGA_W;
-		d->vsize.height = MS_VIDEO_SIZE_VGA_H;
-		d->fps=25;
-	} else if (d->bitrate>=256000){
-		d->vsize.width = MS_VIDEO_SIZE_VGA_W;
-		d->vsize.height = MS_VIDEO_SIZE_VGA_H;
-		d->fps=15;
-	}else if (d->bitrate>=170000){
-		d->vsize.width=MS_VIDEO_SIZE_QVGA_W;
-		d->vsize.height=MS_VIDEO_SIZE_QVGA_H;
-		d->fps=15;
-	}else if (d->bitrate>=128000){
-		d->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		d->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		d->fps=10;
-	}else if (d->bitrate>=64000){
-		d->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		d->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		d->fps=7;
-	}else{
-		d->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		d->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		d->fps=5;
+
+	ms_message("Video configuration set: bitrate=%dbits/s, fps=%f, vsize=%dx%d", d->vconf.required_bitrate, d->vconf.fps, d->vconf.vsize.width, d->vconf.vsize.height);
+	return 0;
+}
+
+static int enc_set_br(MSFilter *f, void *arg) {
+	EncData *d = (EncData *)f->data;
+	int br = *(int *)arg;
+	if (d->enc != NULL) {
+		/* Encoding is already ongoing, do not change video size, only bitrate. */
+		d->vconf.required_bitrate = br;
+		enc_set_configuration(f,&d->vconf);
+	} else {
+		MSVideoConfiguration best_vconf = ms_video_find_best_configuration_for_bitrate(d->vconf_list, br, ms_get_cpu_count());
+		enc_set_configuration(f, &best_vconf);
 	}
-
-#if defined (ANDROID) || TARGET_OS_IPHONE==1
-	d->vsize.width=MS_VIDEO_SIZE_QVGA_W;
-	d->vsize.height=MS_VIDEO_SIZE_QVGA_H;
-	d->fps=12;
-
-#endif
-	
-	ms_message("bitrate requested...: %d (%d x %d)\n", d->bitrate, d->vsize.width, d->vsize.height);
 	return 0;
 }
 
 static int enc_set_fps(MSFilter *f, void *arg){
 	EncData *d=(EncData*)f->data;
-	d->fps=*(float*)arg;
+	d->vconf.fps=*(float*)arg;
+	enc_set_configuration(f, &d->vconf);
 	return 0;
 }
 
 static int enc_get_fps(MSFilter *f, void *arg){
 	EncData *d=(EncData*)f->data;
-	*(float*)arg=d->fps;
+	*(float*)arg=d->vconf.fps;
 	return 0;
 }
 
 static int enc_get_vsize(MSFilter *f, void *arg){
 	EncData *d=(EncData*)f->data;
-	*(MSVideoSize*)arg=d->vsize;
+	*(MSVideoSize*)arg=d->vconf.vsize;
 	return 0;
 }
 
 static int enc_set_vsize(MSFilter *f, void *arg){
-	EncData *d=(EncData*)f->data;
-
-	d->vsize=*(MSVideoSize*)arg;
-
+	MSVideoConfiguration best_vconf;
+	EncData *d = (EncData *)f->data;
+	MSVideoSize *vs = (MSVideoSize *)arg;
+	best_vconf = ms_video_find_best_configuration_for_size(d->vconf_list, *vs, ms_get_cpu_count());
+	d->vconf.vsize = *vs;
+	d->vconf.fps = best_vconf.fps;
+	d->vconf.bitrate_limit = best_vconf.bitrate_limit;
+	enc_set_configuration(f, &d->vconf);
 	return 0;
 }
 
@@ -356,18 +379,27 @@ static int enc_req_vfu(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int enc_get_configuration_list(MSFilter *f, void *data) {
+	EncData *d = (EncData *)f->data;
+	const MSVideoConfiguration **vconf_list = (const MSVideoConfiguration **)data;
+	*vconf_list = d->vconf_list;
+	return 0;
+}
 
-static MSFilterMethod enc_methods[]={
-	{	MS_FILTER_SET_FPS	,	enc_set_fps	},
-	{	MS_FILTER_SET_BITRATE	,	enc_set_br	},
-	{	MS_FILTER_GET_BITRATE	,	enc_get_br	},
-	{	MS_FILTER_GET_FPS	,	enc_get_fps	},
-	{	MS_FILTER_GET_VIDEO_SIZE,	enc_get_vsize	},
-	{	MS_FILTER_SET_VIDEO_SIZE,	enc_set_vsize	},
-	{	MS_FILTER_ADD_FMTP	,	enc_add_fmtp	},
-	{	MS_FILTER_REQ_VFU	,	enc_req_vfu	},
-	{	MS_VIDEO_ENCODER_REQ_VFU,	enc_req_vfu	},
-	{	0	,			NULL		}
+
+static MSFilterMethod enc_methods[] = {
+	{ MS_FILTER_SET_FPS,                       enc_set_fps                },
+	{ MS_FILTER_SET_BITRATE,                   enc_set_br                 },
+	{ MS_FILTER_GET_BITRATE,                   enc_get_br                 },
+	{ MS_FILTER_GET_FPS,                       enc_get_fps                },
+	{ MS_FILTER_GET_VIDEO_SIZE,                enc_get_vsize              },
+	{ MS_FILTER_SET_VIDEO_SIZE,                enc_set_vsize              },
+	{ MS_FILTER_ADD_FMTP,                      enc_add_fmtp               },
+	{ MS_FILTER_REQ_VFU,                       enc_req_vfu                },
+	{ MS_VIDEO_ENCODER_REQ_VFU,                enc_req_vfu                },
+	{ MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, enc_get_configuration_list },
+	{ MS_VIDEO_ENCODER_SET_CONFIGURATION,      enc_set_configuration      },
+	{ 0,                                       NULL                       }
 };
 
 #ifndef _MSC_VER
